@@ -134,16 +134,14 @@ get_visible_text (GtkWidget         *widget,
 }
 
 static gint
-get_vertical_text_height (GtkWidget         *widget)
+get_vertical_text_height (GtkWidget *widget)
 {
-  GcalEventWidget *self;
   GtkBorder border, padding;
   GtkStyleContext *context;
   PangoLayout *layout;
   gint layout_height;
   gchar* display_text;
 
-  self = GCAL_EVENT_WIDGET (widget);
   context = gtk_widget_get_style_context (widget);
   display_text = get_visible_text (widget, GTK_ORIENTATION_VERTICAL);
   layout = gtk_widget_create_pango_layout (widget, display_text);
@@ -298,8 +296,8 @@ gcal_event_widget_set_event_tooltip (GcalEventWidget *self,
             {
               if (is_ltr)
                 {
-                  start = g_date_time_format (tooltip_start, "%x I:%M %P");
-                  end = g_date_time_format (tooltip_end, "%x I:%M %P");
+                  start = g_date_time_format (tooltip_start, "%x %I:%M %P");
+                  end = g_date_time_format (tooltip_end, "%x %I:%M %P");
                 }
               else
                 {
@@ -327,8 +325,8 @@ gcal_event_widget_set_event_tooltip (GcalEventWidget *self,
             {
               if (is_ltr)
                 {
-                  start = g_date_time_format (tooltip_start, "%x, I:%M %P");
-                  end = g_date_time_format (tooltip_end, "I:%M %P");
+                  start = g_date_time_format (tooltip_start, "%x, %I:%M %P");
+                  end = g_date_time_format (tooltip_end, "%I:%M %P");
                 }
               else
                 {
@@ -361,7 +359,9 @@ gcal_event_widget_set_event_tooltip (GcalEventWidget *self,
       escaped_location = g_markup_escape_text (gcal_event_get_location (event), -1);
 
       g_string_append (tooltip_mesg, "\n\n");
-      g_string_append_printf (tooltip_mesg, "%s", escaped_location);
+
+      /* Translators: %s is the location of the event (e.g. "Downtown, 3rd Avenue") */
+      g_string_append_printf (tooltip_mesg, _("At %s"), escaped_location);
     }
 
   description_len = g_utf8_strlen (gcal_event_get_description (event), -1);
@@ -1241,3 +1241,31 @@ gcal_event_widget_compare_by_start_date (GcalEventWidget *widget1,
 {
   return g_date_time_compare (widget1->dt_start, widget2->dt_start);
 }
+
+gint
+gcal_event_widget_sort_events (GcalEventWidget *widget1,
+                               GcalEventWidget *widget2)
+{
+  g_autoptr (GDateTime) dt_time1, dt_time2;
+  icaltimetype *ical_dt;
+  gint diff;
+
+  dt_time1 = dt_time2 = NULL;
+  diff = gcal_event_widget_compare_by_start_date (widget1, widget2);
+
+  if (diff != 0)
+    return diff;
+
+  diff = gcal_event_widget_compare_by_length (widget1, widget2);
+
+  if (diff != 0)
+    return diff;
+
+  e_cal_component_get_last_modified (gcal_event_get_component (widget1->event), &ical_dt);
+  dt_time1 = icaltime_to_datetime (ical_dt);
+
+  e_cal_component_get_last_modified (gcal_event_get_component (widget2->event), &ical_dt);
+  dt_time2 = icaltime_to_datetime (ical_dt);
+
+  return g_date_time_compare (dt_time2, dt_time1);
+}
