@@ -39,6 +39,48 @@
 #include <libecal/libecal.h>
 #include <libical/icaltime.h>
 
+/**
+ * SECTION:gcal-window
+ * @short_description: Main window of GNOME Calendar
+ * @title:GcalWindow
+ * @stability:unstable
+ * @image:gcal-window.png
+ *
+ * #GcalWindow is the main window of GNOME Calendar, and contains
+ * the views, the source dialog, the edit dialog, and manages the
+ * calendar toggler popover menu.
+ *
+ * Besides that, #GcalWindow is also responsible for #GcalQuickAddPopover,
+ * and it responds to the #GcalView:create-event signal by positioning
+ * the quick add popover at the requested position.
+ *
+ * ## Calendar popover
+ *
+ * ![The popover](gcal-window_agendas.png)
+ *
+ * The calendar popover enables/disables the selected calendars.
+ * This is simply an UI for gcal_manager_enable_source() and
+ * gcal_manager_disable_source().
+ *
+ * The calendar popover also contains a button to open the source
+ * dialog.
+ *
+ * ## Edit dialog
+ *
+ * When an event is clicked, the views send the `event-activated`
+ * signal. #GcalWindow responds to this signal opening #GcalEditDialog
+ * with the clicked event.
+ *
+ * When #GcalEditDialog sends a response, #GcalWindow reacts by
+ * either propagating to gcal_manager_update_event(), or hiding
+ * the delete event widgets from the views.
+ *
+ * ## Source dialog
+ *
+ * The interaction with the source dialog is almost none. #GcalWindow
+ * just shows and hides it.
+ */
+
 typedef struct
 {
   gint               x;
@@ -107,7 +149,8 @@ struct _GcalWindow
 
   NewEventData        *event_creation_data;
 
-  GcalEvent           *event_to_delete;
+  GcalEvent             *event_to_delete;
+  GcalRecurrenceModType  event_to_delete_mod;
 
   /* calendar management */
   GtkWidget           *calendar_popover;
@@ -479,7 +522,6 @@ calendar_listbox_sort_func (GtkListBoxRow *row1,
 static void
 load_geometry (GcalWindow *window)
 {
-  GcalApplication *app;
   GSettings *settings;
   GVariant *variant;
   gboolean maximized;
@@ -489,8 +531,7 @@ load_geometry (GcalWindow *window)
 
   GCAL_ENTRY;
 
-  app = GCAL_APPLICATION (gtk_window_get_application (GTK_WINDOW (window)));
-  settings = gcal_application_get_settings (app);
+  settings = gcal_manager_get_settings (window->manager);
 
   /* load window settings: size */
   variant = g_settings_get_value (settings,
@@ -533,7 +574,6 @@ save_geometry (gpointer user_data)
   GtkWindow *self;
   GdkWindow *win;
   GdkWindowState state;
-  GcalApplication *app;
   GSettings *settings;
   gboolean maximized;
   GVariant *variant;
@@ -546,10 +586,7 @@ save_geometry (gpointer user_data)
   window = GCAL_WINDOW (self);
   win = gtk_widget_get_window (GTK_WIDGET (self));
   state = gdk_window_get_state (win);
-
-
-  app = GCAL_APPLICATION (gtk_window_get_application (self));
-  settings = gcal_application_get_settings (app);
+  settings = gcal_manager_get_settings (window->manager);
 
   /* save window's state */
   maximized = state & GDK_WINDOW_STATE_MAXIMIZED;
@@ -1038,11 +1075,13 @@ edit_dialog_closed (GtkDialog *dialog,
                     gint       response,
                     gpointer   user_data)
 {
+  GcalRecurrenceModType mod;
   GcalWindow *window;
   GcalEditDialog *edit_dialog;
   GcalEvent *event;
   GcalView *view;
   GList *widgets;
+  ESource *source;
 
   GCAL_ENTRY;
 
@@ -1050,8 +1089,20 @@ edit_dialog_closed (GtkDialog *dialog,
   edit_dialog = GCAL_EDIT_DIALOG (dialog);
   event = gcal_edit_dialog_get_event (edit_dialog);
   view = GCAL_VIEW (window->views[window->active_view]);
+  mod = GCAL_RECURRENCE_MOD_THIS_ONLY;
+  source = gcal_event_get_source (event);
+
+  if (!gcal_edit_dialog_get_recurrence_changed (edit_dialog) &&
+      gcal_event_has_recurrence (event) &&
+      (response != GCAL_RESPONSE_CREATE_EVENT &&
+       response != GTK_RESPONSE_CANCEL &&
+       response != GTK_RESPONSE_DELETE_EVENT &&
+       gcal_event_has_recurrence (event) &&
+       !ask_recurrence_modification_type (GTK_WIDGET (dialog), &mod, source)))
+    {
+      return;
+    }
 
-  gtk_widget_hide (GTK_WIDGET (dialog));
 
   switch (response)
     {
@@ -1060,13 +1111,13 @@ edit_dialog_closed (GtkDialog *dialog,
       break;
 
     case GCAL_RESPONSE_SAVE_EVENT:
-      gcal_manager_update_event (window->manager, event);
+      gcal_manager_update_event (window->manager, event, mod);
       break;
 
     case GCAL_RESPONSE_DELETE_EVENT:
       if (window->event_to_delete != NULL)
         {
-          gcal_manager_remove_event (window->manager, window->event_to_delete);
+          gcal_manager_remove_event (window->manager, window->event_to_delete, window->event_to_delete_mod);
           g_clear_object (&window->event_to_delete);
 
           create_notification (GCAL_WINDOW (user_data), _("Another event deleted"), _("Undo"));
@@ -1085,8 +1136,10 @@ edit_dialog_closed (GtkDialog *dialog,
 
       g_set_object (&window->event_to_delete, event);
 
+      window->event_to_delete_mod = mod;
+
       /* hide widget of the event */
-      widgets = gcal_view_get_children_by_uuid (view, gcal_event_get_uid (event));
+      widgets = gcal_view_get_children_by_uuid (view, mod, gcal_event_get_uid (event));
 
       g_list_foreach (widgets, (GFunc) gtk_widget_hide, NULL);
       g_list_free (widgets);
@@ -1098,6 +1151,8 @@ edit_dialog_closed (GtkDialog *dialog,
 
     }
 
+  gtk_widget_hide (GTK_WIDGET (dialog));
+
   gcal_edit_dialog_set_event (edit_dialog, NULL);
 
   GCAL_EXIT;
@@ -1167,7 +1222,7 @@ remove_event (GtkWidget  *notification,
 
   if (window->event_to_delete != NULL)
     {
-      gcal_manager_remove_event (window->manager, window->event_to_delete);
+      gcal_manager_remove_event (window->manager, window->event_to_delete, window->event_to_delete_mod);
       g_clear_object (&window->event_to_delete);
     }
 }
@@ -1181,6 +1236,7 @@ undo_remove_action (GtkButton *button,
 
   window = GCAL_WINDOW (user_data);
   widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (window->views[window->active_view]),
+                                            window->event_to_delete_mod,
                                             gcal_event_get_uid (window->event_to_delete));
 
   /* Show the hidden to-be-deleted events */
@@ -1202,7 +1258,9 @@ schedule_open_edit_dialog_by_uuid (OpenEditDialogData *edit_dialog_data)
   GList *widgets;
 
   window = edit_dialog_data->window;
-  widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (window->month_view), edit_dialog_data->uuid);
+  widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (window->month_view),
+                                            GCAL_RECURRENCE_MOD_THIS_ONLY,
+                                            edit_dialog_data->uuid);
   if (widgets != NULL)
     {
       window->open_edit_dialog_timeout_id = 0;
@@ -1225,26 +1283,18 @@ gcal_window_finalize (GObject *object)
 
   GCAL_ENTRY;
 
-  if (window->save_geometry_timeout_id > 0)
-    {
-      g_source_remove (window->save_geometry_timeout_id);
-      window->save_geometry_timeout_id = 0;
-    }
-
-  if (window->open_edit_dialog_timeout_id > 0)
-    {
-      g_source_remove (window->open_edit_dialog_timeout_id);
-      window->open_edit_dialog_timeout_id = 0;
-    }
+  gcal_clear_timeout (&window->save_geometry_timeout_id);
+  gcal_clear_timeout (&window->open_edit_dialog_timeout_id);
+  gcal_clear_timeout (&window->refresh_timeout_id);
 
   /* If we have a queued event to delete, remove it now */
   if (window->event_to_delete)
     {
-      gcal_manager_remove_event (window->manager, window->event_to_delete);
+      gcal_manager_remove_event (window->manager, window->event_to_delete, window->event_to_delete_mod);
       g_clear_object (&window->event_to_delete);
     }
 
-  if (window->event_creation_data != NULL)
+  if (window->event_creation_data)
     {
       g_clear_pointer (&window->event_creation_data->start_date, g_date_time_unref);
       g_clear_pointer (&window->event_creation_data->end_date, g_date_time_unref);
@@ -1254,7 +1304,7 @@ gcal_window_finalize (GObject *object)
   g_clear_object (&window->manager);
   g_clear_object (&window->views_switcher);
 
-  g_free (window->active_date);
+  g_clear_pointer (&window->active_date, g_free);
 
   G_OBJECT_CLASS (gcal_window_parent_class)->finalize (object);
 
@@ -1291,6 +1341,11 @@ gcal_window_set_property (GObject      *object,
     case PROP_MANAGER:
       if (g_set_object (&self->manager, g_value_get_object (value)))
         {
+          g_settings_bind (gcal_manager_get_settings (self->manager),
+                           "active-view",
+                           self,
+                           "active-view",
+                           G_SETTINGS_BIND_SET | G_SETTINGS_BIND_GET);
 
           if (!gcal_manager_get_loading (self->manager))
             {
@@ -1309,12 +1364,6 @@ gcal_window_set_property (GObject      *object,
           g_signal_connect_swapped (self->manager, "source-enabled", G_CALLBACK (source_enabled), object);
           g_signal_connect_swapped (self->manager, "source-changed", G_CALLBACK (source_changed), object);
 
-          gcal_edit_dialog_set_manager (GCAL_EDIT_DIALOG (self->edit_dialog), self->manager);
-          gcal_week_view_set_manager (GCAL_WEEK_VIEW (self->week_view), self->manager);
-          gcal_month_view_set_manager (GCAL_MONTH_VIEW (self->month_view), self->manager);
-          gcal_year_view_set_manager (GCAL_YEAR_VIEW (self->year_view), self->manager);
-          gcal_quick_add_popover_set_manager (GCAL_QUICK_ADD_POPOVER (self->quick_add_popover), self->manager);
-          gcal_source_dialog_set_manager (GCAL_SOURCE_DIALOG (self->source_dialog), self->manager);
           gcal_search_view_connect (GCAL_SEARCH_VIEW (self->search_view), self->manager);
 
           g_object_notify (object, "manager");
@@ -1408,11 +1457,19 @@ gcal_window_state_event (GtkWidget           *widget,
 }
 
 static void
-gcal_window_class_init(GcalWindowClass *klass)
+gcal_window_class_init (GcalWindowClass *klass)
 {
   GObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
+  g_type_ensure (GCAL_TYPE_EDIT_DIALOG);
+  g_type_ensure (GCAL_TYPE_MONTH_VIEW);
+  g_type_ensure (GCAL_TYPE_QUICK_ADD_POPOVER);
+  g_type_ensure (GCAL_TYPE_SEARCH_VIEW);
+  g_type_ensure (GCAL_TYPE_SOURCE_DIALOG);
+  g_type_ensure (GCAL_TYPE_WEEK_VIEW);
+  g_type_ensure (GCAL_TYPE_YEAR_VIEW);
+
   object_class = G_OBJECT_CLASS (klass);
   object_class->finalize = gcal_window_finalize;
   object_class->set_property = gcal_window_set_property;
@@ -1421,7 +1478,6 @@ gcal_window_class_init(GcalWindowClass *klass)
   widget_class = GTK_WIDGET_CLASS (klass);
   widget_class->configure_event = gcal_window_configure_event;
   widget_class->window_state_event = gcal_window_state_event;
-  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/window.ui");
 
   g_object_class_install_property (
       object_class,
@@ -1438,7 +1494,7 @@ gcal_window_class_init(GcalWindowClass *klass)
       PROP_MANAGER,
       g_param_spec_object ("manager",
                            "The manager object",
-                           "A weak reference to the app manager object",
+                           "The manager object",
                            GCAL_TYPE_MANAGER,
                            G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
 
@@ -1460,6 +1516,8 @@ gcal_window_class_init(GcalWindowClass *klass)
                             FALSE,
                             G_PARAM_READWRITE));
 
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/window.ui");
+
   /* widgets */
   gtk_widget_class_bind_template_child (widget_class, GcalWindow, edit_dialog);
   gtk_widget_class_bind_template_child (widget_class, GcalWindow, header_bar);
@@ -1589,6 +1647,17 @@ gcal_window_init (GcalWindow *self)
   self->active_date = g_new0 (icaltimetype, 1);
   self->rtl = gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL;
 
+  /*
+   * FIXME: this is a hack around the issue that happens when trying to bind
+   * there properties using the GtkBuilder .ui file.
+   */
+  g_object_bind_property (self, "manager", self->edit_dialog, "manager", G_BINDING_DEFAULT);
+  g_object_bind_property (self, "manager", self->source_dialog, "manager", G_BINDING_DEFAULT);
+  g_object_bind_property (self, "manager", self->week_view, "manager", G_BINDING_DEFAULT);
+  g_object_bind_property (self, "manager", self->month_view, "manager", G_BINDING_DEFAULT);
+  g_object_bind_property (self, "manager", self->year_view, "manager", G_BINDING_DEFAULT);
+  g_object_bind_property (self, "manager", self->quick_add_popover, "manager", G_BINDING_DEFAULT);
+
   /* setup accels */
   app = g_application_get_default ();
 
@@ -1604,10 +1673,18 @@ gcal_window_init (GcalWindow *self)
 }
 
 /* Public API */
+/**
+ * gcal_window_new_with_date:
+ * @app: a #GcalApplication
+ * @date: the active date
+ *
+ * Creates a #GcalWindow, positions it at @date.
+ *
+ * Returns: (transfer full): a #GcalWindow
+ */
 GtkWidget*
-gcal_window_new_with_view_and_date (GcalApplication    *app,
-                                    GcalWindowViewType  view_type,
-                                    icaltimetype       *date)
+gcal_window_new_with_date (GcalApplication *app,
+                           icaltimetype    *date)
 {
   GcalManager *manager;
   GcalWindow *win;
@@ -1622,13 +1699,16 @@ gcal_window_new_with_view_and_date (GcalApplication    *app,
   /* loading size */
   load_geometry (win);
 
-  if (view_type == GCAL_WINDOW_VIEW_DAY)
-    view_changed (NULL, NULL, win);
-
   return GTK_WIDGET (win);
 }
 
 /* new-event interaction: first variant */
+/**
+ * gcal_window_new_event:
+ * @self: a #GcalWindow
+ *
+ * Makes #GcalWindow create a new event.
+ */
 void
 gcal_window_new_event (GcalWindow *self)
 {
@@ -1660,6 +1740,15 @@ gcal_window_new_event (GcalWindow *self)
   create_event_detailed_cb (NULL, start_date, end_date, self);
 }
 
+/**
+ * gcal_window_set_search_mode:
+ * @self: a #GcalWindow
+ * @enabled: whether the search mode is enabled or not
+ *
+ * Sets whether #GcalWindow is in search mode. This is used by
+ * #GcalShellSearchProvider to respond to the user clicking on
+ * GNOME Calendar icon at the search.
+ */
 void
 gcal_window_set_search_mode (GcalWindow *self,
                              gboolean    enabled)
@@ -1670,6 +1759,15 @@ gcal_window_set_search_mode (GcalWindow *self,
   gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (self->search_bar), enabled);
 }
 
+/**
+ * gcal_window_set_search_query:
+ * @self: a #GcalWindow
+ * @query: sets the search query of @self
+ *
+ * Sets the search query of the search. #GcalWindow only
+ * propagates this to the search bar, which ends up
+ * triggering the search.
+ */
 void
 gcal_window_set_search_query (GcalWindow  *self,
                               const gchar *query)
@@ -1679,6 +1777,14 @@ gcal_window_set_search_query (GcalWindow  *self,
   gtk_entry_set_text (GTK_ENTRY (self->search_entry), query);
 }
 
+/**
+ * gcal_window_open_event_by_uuid:
+ * @self: a #GcalWindow
+ * @uuid: the unique identifier of the event to be opened
+ *
+ * Tells @self to open the event with @uuid. When it fails to
+ * open the event, it waits for 2 seconds before trying again.
+ */
 void
 gcal_window_open_event_by_uuid (GcalWindow  *self,
                                 const gchar *uuid)
@@ -1688,7 +1794,9 @@ gcal_window_open_event_by_uuid (GcalWindow  *self,
   /* XXX: show events on month view */
   gtk_stack_set_visible_child (GTK_STACK (self->views_stack), self->month_view);
 
-  widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (self->month_view), uuid);
+  widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (self->month_view),
+                                            GCAL_RECURRENCE_MOD_THIS_ONLY,
+                                            uuid);
 
   if (widgets)
     {
